# ruby dependencies
require 'logger'
require 'fileutils'
require 'yaml'
require 'singleton'

# external dependencies
require 'amqp'
require 'confstruct'
require 'kgio'
require 'active_support/core_ext/module/delegation'
require 'active_support/core_ext/hash/reverse_merge'
require 'active_support/concern'
require 'active_support/inflections'
require 'awesome_print'
require 'awesome_print/core_ext/logger'
require 'msgpack'

# kinetic dependencies
require_relative '../kinetic/version'
require_relative '../kinetic/configuration'
require_relative '../kinetic/master'
require_relative '../kinetic/worker'
require_relative '../kinetic/dsl'
require_relative '../kinetic/errors'
require_relative '../kinetic/publisher'

module Kinetic

  module Delegator

    Kinetic::DSL.public_instance_methods.each.each { |m| delegate m, to: :application }

    def application
      Application.instance
    end


  end

  class Base
    include Singleton
    include Kinetic::DSL
    
    def initialize
      logger.info "Kinetic #{Kinetic::VERSION}"
      logger.debug "Loading serializer #{config.serializer}"
      require "kinetic/serializers/#{config.serializer.downcase}"
      @serializer = Object.const_get("Kinetic::Serializers::#{config.serializer}")
    end

    private

    def direct
      exchanges[:direct] ||= {}
    end

    def after_fork_procs
      @after_fork_procs ||= []
    end

    def exchanges
      @exchanges ||= {}
    end

    def timed_events
      @timed_events ||= {}
    end

  end

end