require_relative 'dsl/configuration'
require_relative 'dsl/timed_events'
require_relative 'dsl/serialization'
module Kinetic

  module DSL

    include Configuration
    include TimedEvents
    include Serialization

    def after_fork(&block)
      raise Kinetic::Errors::BlockMissing unless block_given?
      after_fork_procs << block
    end


    # Defines a direct queue subscription. Direct queues do not allow fuzzy matching so all messages sent to this queue
    # must exactly match the key.
    #
    # @param [String] key the key of the queue to which to subscribe.
    #
    # @yield [message] yields the message passed to the queue to the block
    #
    # @raise [Kinetic::Errors::NoSubcriberBlock] if a block is not passed
    # @raise [Kinetic::Errors::KeyMustBeString]  if the passed key is not a string
    def on_direct(key, &block)
      raise Kinetic::Errors::NoSubscriberBlock unless block_given?
      raise Kinetic::Errors::KeyMustBeString   unless key.is_a? String
      logger.debug "Setting up '#{key}' on 'direct'"
      direct[key] = block
    end

    alias :on :on_direct

    # @return [Logger] returns the application logger instance
    def logger
      @logger ||= config[:logging] ? reopen_logger : Logger.new('/dev/null')
    end

    protected

    def reopen_logger
      @logger = Logger.new(config[:daemonize] ? config.log_file : STDOUT)
    end



  end

end