require 'logger'
require 'bunny'
require 'singleton'
require 'active_support/core_ext/module/delegation'


require_relative 'dsl/configuration'
require_relative './configuration'

module Kinetic

  class Publisher
    include Kinetic::DSL::Configuration
    include Singleton

    class << self

      def method_missing(meth, *args, &block)
        self.instance.send(meth, *args, &block)
      end

    end

    def initialize
      logger.debug "Using serializer #{config.serializer}"
      require_relative "./serializers/#{config.serializer.to_s.downcase}"
      @serializer = Kinetic::Serializers.const_get(config.serializer)
   	end

    def publish_topic(key, message)
      topic_exchange.publish(serialize(message), routing_key: key)
      true
    end

    def publish_fanout(key, message)
      fanout_exchange.publish(serialize(message), routing_key: key)
    end

    def publish_direct(key, message)
      direct_exchange.publish(serialize(message), routing_key: key)
    end

    alias :publish :publish_direct

    private

    def topic_exchange
    	exchange(:topic)
    end

    def fanout_exchange
    	exchange(:fanout)
    end

    def direct_exchange
      exchange(:direct)
    end

    def serialize(message)
      @serializer.serialize(message)
    end

    def deserialize(message)
      @serializer.deserialize(message)
    end

    private

    def exchange(type)
    	Bunny::Exchange.new(channel, type, "#{config.name}.#{type}")
    end


    def channel
      unless @connection
        logger.info "Establishing publisher connection to amqp://#{config.host}:#{config.port}"
        @connection = Bunny.new(host: config.host, port: config.port, threaded: false)
        logger.debug 'Starting connection'
        @connection.start
        @channel    = @connection.channel
      end
      @channel
    end

    def logger
      @logger ||= Logger.new(File.join(config.root, config.log_file))
   	end


  end

end
